@@ -2,7 +2,7 @@
 ** Copyright 1998 - 2004 Double Precision, Inc.
 ** See COPYING for distribution information.
 **
-** $Id: pop3dserver.c,v 1.30 2004/05/09 02:52:23 mrsam Exp $
+** $Id: pop3dserver.c,v 1.37 2005/03/01 03:11:24 mrsam Exp $
 */
 
 #if	HAVE_CONFIG_H
@@ -48,8 +48,6 @@
 #endif
 #include	<signal.h>
 #include	<errno.h>
-#include	"authlib/auth.h"
-#include	"authlib/authmod.h"
 #include	"numlib/numlib.h"
 #include	"maildir/config.h"
 #include	"maildir/maildirmisc.h"
@@ -57,6 +55,7 @@
 #include	"maildir/maildirgetquota.h"
 #include	"maildir/maildircreate.h"
 #include	"maildir/loginexec.h"
+#include	"courierauth.h"
 
 #define POP3DLIST "courierpop3dsizelist"
 
@@ -110,6 +109,8 @@ static unsigned long retr_count=0;
 static unsigned long uidv=0;
 static int convert_v0=0;
 
+static time_t start_time;
+
 /*
 ** The RFC is pretty strict in stating that octet size must count the CR
 ** in the CRLF endofline.
@@ -396,7 +397,7 @@ static int cmpmsgs(const void *a, const void *b)
 
 	while (*ap || *bp)
 	{
-		if (*ap == ':' && *bp == ':')
+		if (*ap == MDIRSEP[0] && *bp == MDIRSEP[0])
 			break;
 
 		if (*ap < *bp)
@@ -778,15 +779,22 @@ static void acctout(const char *disc)
 	static const char msg3[]=", ip=[";
 	static const char msg4[]="], top=";
 	static const char msg5[]=", retr=";
+	static const char msg6[]=", time=";
+	static const char msg7[]=", stls=1";
+
 	char num1[NUMBUFSIZE];
 	char num2[NUMBUFSIZE];
+	char num3[NUMBUFSIZE];
+
 	char *p;
+	const char *q;
 
 	libmail_str_size_t(top_count, num1);
 	libmail_str_size_t(retr_count, num2);
+	libmail_str_time_t(time(NULL)-start_time, num3);
 
 	p=malloc(strlen(authaddr)+strlen(remoteip)+strlen(disc)+
-		 strlen(num1)+strlen(num2)+200);	/* Should be enough */
+		 strlen(num1)+strlen(num2)+strlen(num3)+200);	/* Should be enough */
 
 	strcpy(p, disc);
 	strcat(p, msg2);
@@ -797,6 +805,12 @@ static void acctout(const char *disc)
 	strcat(p, num1);
 	strcat(p, msg5);
 	strcat(p, num2);
+	strcat(p, msg6);
+	strcat(p, num3);
+
+	if ((q=getenv("POP3_STLS")) && atoi(q))
+		strcat(p, msg7);
+
 	strcat(p, "\n");
 	write(2, p, strlen(p));
 	free(p);
@@ -804,7 +818,7 @@ static void acctout(const char *disc)
 
 static RETSIGTYPE bye(int signum)
 {
-	acctout("ERR: TIMEOUT");
+	acctout("INFO: TIMEOUT");
 	exit(0);
 #if	RETSIGTYPE != void
 	return (0);
@@ -925,7 +939,7 @@ error:
 		printf("-ERR Invalid command.\r\n");
 		fflush(stdout);
 	}
-	acctout("ERR: DISCONNECTED");
+	acctout("INFO: DISCONNECTED");
 }
 
 /* Like every good Maildir reader, we purge the tmp subdirectory */
@@ -962,16 +976,19 @@ int main(int argc, char **argv)
 {
 char	*p;
 
-	authmodclient();
+	time(&start_time);
 
-	if ((authaddr=getenv("AUTHADDR")) == 0 ||
-		(remoteip=getenv("TCPREMOTEIP")) == 0)
+	if ((authaddr=getenv("AUTHADDR")) == NULL ||
+	    *authaddr == 0)
 	{
-		fprintf(stderr, "ERROR: Required environment variables not initialized.\n");
-		fflush(stderr);
-		exit (1);
+		authaddr=getenv("AUTHENTICATED");
+		if (authaddr == NULL || *authaddr == 0)
+			authaddr="nobody";
 	}
 
+	if ((remoteip=getenv("TCPREMOTEIP")) == NULL)
+		remoteip="127.0.0.1";
+
 	{
 	struct	stat	buf;
 
@@ -984,34 +1001,31 @@ char	*p;
 		}
 	}
 
-	if ((p=getenv("MAILDIR")) != 0 && *p)
+	if (argc > 1)
+		p=argv[1];
+	else
+		p=getenv("MAILDIR");
+
+	if (!p)
+		p="./Maildir";
+
+	if (chdir(p))
 	{
-		if (chdir(p))
-		{
-			printf("-ERR Maildir: %s\r\n",strerror(errno));
-			exit(1);
-		}
-	}
-	else if (argc > 1 && chdir(argv[1]))
-	{
-		printf("-ERR Maildir: %s\r\n",strerror(errno));
+		fprintf(stderr, "chdir %s: %s\n", p, strerror(errno));
+		printf("-ERR chdir %s failed\n", p);
+		fflush(stdout);
 		exit(1);
 	}
 	
 	maildir_loginexec();
 
-	p=authgetoptionenv("disablepop3");
-
-	if (p && atoi(p))
+	if (auth_getoptionenvint("disablepop3"))
 	{
 		printf("-ERR POP3 access disabled for this account.\r\n");
 		fflush(stdout);
 		exit(1);
 	}
 
-	free(p);
-
-
 	fprintf(stderr, "INFO: LOGIN, user=%s, ip=[%s]\n",
 			authaddr,
 			remoteip);
