@@ -44,7 +44,6 @@
 #include	"mailboxlist.h"
 #include	"thread.h"
 #include	"outbox.h"
-#include	"authlib/authmod.h"
 
 #include	"imapwrite.h"
 #include	"imaptoken.h"
@@ -67,7 +66,7 @@
 
 #define SMAP_BUFSIZ 8192
 
-static const char rcsid[]="$Id: smap.c,v 1.22 2004/02/15 17:13:07 mrsam Exp $";
+static const char rcsid[]="$Id: smap.c,v 1.26 2005/07/16 03:10:59 mrsam Exp $";
 #define SHARED "shared"
 
 #define LIST_FOLDER 1
@@ -366,28 +365,21 @@ static void list_callback(const char *f, void *vp)
 
 	if (maildir_acl_read(&l, utf8->homedir, strchr(f, '.')) == 0)
 	{
-		const char *owner_array[3];
-		maildir_aclt aa;
-		char *o;
+		char *myrights;
+		char *owner=malloc(sizeof("user=")+strlen(utf8->owner));
 
-		owner_array[0]=o=malloc(sizeof("user=")+
-					strlen(getenv("AUTHENTICATED")));
-		if (o == NULL)
+		if (!owner)
 			write_error_exit(0);
-		strcat(strcpy(o, "user="), getenv("AUTHENTICATED"));
-		owner_array[1]= strcmp(utf8->owner, getenv("AUTHENTICATED"))
-			? NULL:"owner";
-		owner_array[2]=NULL;
 
-		if (maildir_acl_compute_array(&aa, &l, owner_array) == 0)
-		{
-			if (strchr(maildir_aclt_ascstr(&aa),
-				   ACL_LOOKUP[0]) != NULL)
-				(*utf8->callback_func)(f, fn,
-						       utf8->callback_arg);
-			maildir_aclt_destroy(&aa);
-		}
-		free(o);
+		strcat(strcpy(owner, "user="), utf8->owner);
+		myrights=compute_myrights(&l, owner);
+		free(owner);
+
+		if (myrights && strchr(myrights, ACL_LOOKUP[0]) != NULL)
+			(*utf8->callback_func)(f, fn, utf8->callback_arg);
+		if (myrights)
+			free(myrights);
+
 		maildir_aclt_list_destroy(&l);
 	}
 	maildir_smapfn_free(fn);
@@ -1488,9 +1480,13 @@ static int dump_hdrs(int fd, unsigned long n,
 	int rc;
         off_t start_pos, end_pos, dummy, start_body;
 	off_t nbodylines;
+	int get_flags=RFC2045H_NOLC;
 
 	rc=0;
 
+	if (type && strcmp(type, "RAWHEADERS") == 0)
+		get_flags |= RFC2045H_KEEPNL;
+
 	if (!rfcp)
 	{
 		struct stat stat_buf;
@@ -1536,7 +1532,7 @@ static int dump_hdrs(int fd, unsigned long n,
 	h=rfc2045header_start(fd, rfcp);
  
 	while (h &&
-	       (rc=rfc2045header_get(h, &header, &value, RFC2045H_NOLC)) == 0
+	       (rc=rfc2045header_get(h, &header, &value, get_flags)) == 0
 	       && header)
 	{
 		if (hashdr(hdrs, header))
@@ -1935,7 +1931,8 @@ static int do_fetch(unsigned long n, void *vp)
 		rc=mime(fd, n, rfcp, fi->hdrs);
 		close(fd);
 	}
-	else if (strcmp(fi->entity, "HEADERS") == 0)
+	else if (strcmp(fi->entity, "HEADERS") == 0 ||
+		 strcmp(fi->entity, "RAWHEADERS") == 0)
 	{
 		int fd;
 		struct rfc2045 *rfcp;
@@ -1960,7 +1957,7 @@ static int do_fetch(unsigned long n, void *vp)
 			}
 		}
 
-		rc=dump_hdrs(fd, n, rfcp, fi->hdrs, "HEADERS");
+		rc=dump_hdrs(fd, n, rfcp, fi->hdrs, fi->entity);
 		close(fd);
 	}
 	else if (strcmp(fi->entity, "BODY") == 0
@@ -2329,7 +2326,7 @@ static void fixnewfilename(char *p)
 
 	q=strrchr(strrchr(p, '/'), MDIRSEP[0]);
 
-	if (strcmp(q, ":2,") == 0)
+	if (strcmp(q, MDIRSEP "2,") == 0)
 	{
 		*q=0;
 		memcpy(strrchr(p, '/')-3, "new", 3);
@@ -3131,7 +3128,7 @@ static int dosetdeleteacl(void *cb_arg, int dodelete)
 			aclminimum(err_failedrights);
 			writes("\n");
 		}
-		writes("+ERR ACL update failed\n");
+		writes("-ERR ACL update failed\n");
 		maildir_aclt_list_destroy(&aclt_list);
 		return 0;
 	}
