@@ -1,5 +1,5 @@
 /*
-** Copyright 1998 - 2002 Double Precision, Inc.
+** Copyright 1998 - 2005 Double Precision, Inc.
 ** See COPYING for distribution information.
 */
 
@@ -16,9 +16,6 @@
 */
 
 #include	"soxwrap/soxwrap.h"
-#ifdef	getc
-#undef	getc
-#endif
 #include	<stdio.h>
 
 
@@ -55,9 +52,10 @@
 
 #include	<netdb.h>
 
-static const char rcsid[]="$Id: tcpd.c,v 1.34 2003/01/05 04:01:18 mrsam Exp $";
+static const char rcsid[]="$Id: tcpd.c,v 1.41 2005/07/02 01:13:57 mrsam Exp $";
 
 static const char *accessarg=0;
+static const char *accesslocal=0;
 static const char *denymsgarg=0;
 static const char *listenarg=0;
 static const char *ipaddrarg=0;
@@ -83,6 +81,7 @@ static void setup_block(const char *);
 
 static struct args arginfo[]={
 	{"access", &accessarg},
+	{"accesslocal", &accesslocal},
 	{"denymsg", &denymsgarg},
 	{"address", &ipaddrarg},
 	{"block", 0, setup_block},
@@ -228,7 +227,7 @@ static int parseaddr(const char *p)
 	char *buf=strdup(p);
 	char *q, *a, *s;
 
-	if (!p)
+	if (!buf)
 	{
 		perror("malloc");
 		return (-1);
@@ -404,7 +403,7 @@ static int mksocket(const char *ipaddrarg,	/* Host/IP address */
 
 	if (sox_bind(fd, (struct sockaddr *)sinaddr, sinaddrlen) < 0)
 	{
-		if (flags && MKS_ERROK)
+		if (flags & MKS_ERROK)
 		{
 			close(fd);
 			return (-2);
@@ -443,7 +442,21 @@ static int mksockets()
 
 	for (p=fdlist; p; p=p->next)
 	{
-		int fd=mksocket(p->ipaddr, p->servname, 0);
+		int fd;
+		struct in_addr addr;
+		int fd_flag=0;
+
+		if (p->ipaddr && strcmp(p->ipaddr, "0"))
+		{
+			/* FreeBSD needs AF_INET binds for IPv4 addys */
+
+			if (rfc1035_aton_ipv4(p->ipaddr, &addr) == 0)
+			{
+				fd_flag=MKS_USEAFINET4;
+			}
+		}
+
+		fd=mksocket(p->ipaddr, p->servname, fd_flag);
 
 		if (fd < 0)
 			break;
@@ -498,23 +511,26 @@ int	i;
 gid_t	gid=0;
 const	char *servname;
 int	forced=0;
-int	lockfd;
+int	lockfd=-1;
  
 	argn=argparse(argc, argv, arginfo);
 
-	if (pidarg == 0)
+	if ((stoparg || restartarg) && pidarg == 0)
 	{
 		fprintf(stderr, "%s: -pid argument is required.\n", argv[0]);
 		return (-1);
 	}
 
-	lockfilename=malloc(strlen(pidarg)+sizeof(".lock"));
-	if (!lockfilename)
+	if (pidarg)
 	{
-		perror("malloc");
-		return (-1);
+		lockfilename=malloc(strlen(pidarg)+sizeof(".lock"));
+		if (!lockfilename)
+		{
+			perror("malloc");
+			return (-1);
+		}
+		strcat(strcpy(lockfilename, pidarg), ".lock");
 	}
-	strcat(strcpy(lockfilename, pidarg), ".lock");
 
 	if (stoparg)
 	{
@@ -535,11 +551,13 @@ int	lockfd;
 		return (-1);
 	}
 
-	lockfd=ll_daemon_start(lockfilename);
-	if (lockfd < 0)
-	{
-		perror("ll_daemon_start");
-		return (-1);
+	if (pidarg) { /* -start implied for backwards compatibility */
+		lockfd=ll_daemon_start(lockfilename);
+		if (lockfd < 0)
+		{
+			perror("ll_daemon_start");
+			return (-1);
+		}
 	}
 
 	servname=argv[argn++];
@@ -596,7 +614,8 @@ int	lockfd;
 	}
 #endif
 
-	ll_daemon_started(pidarg, lockfd);
+	if (pidarg)
+		ll_daemon_started(pidarg, lockfd);
 
 	if (grouparg)
 	{
@@ -631,7 +650,7 @@ int	lockfd;
 		}
 	}
 
-	if (ll_daemon_resetio())
+	if (pidarg && ll_daemon_resetio())
 	{
 		perror("ll_daemon_resetio");
 		close(lockfd);
@@ -804,22 +823,22 @@ static RETSIGTYPE childsig(int signum)
 #endif
 }
 
-static int doallowaccess(char *);
+static int doallowaccess(char *, int);
 
 #if	RFC1035_IPV6
 
-static int allowaccess(const RFC1035_ADDR *sin)
+static int allowaccess(const RFC1035_ADDR *sin, int port)
 {
-char	buf[RFC1035_MAXNAMESIZE+1];
+char	buf[RFC1035_MAXNAMESIZE+1+6];
 char	*q;
 int	i;
 
 	if (IN6_IS_ADDR_V4MAPPED(sin))
 	{
-	const char *p=inet_ntop(AF_INET6, sin, buf, sizeof(buf));
+	const char *p=inet_ntop(AF_INET6, sin, buf, sizeof(buf)-6);
 
 		if (p && (q=strrchr(buf, ':')) != 0)
-			return (doallowaccess(q+1));
+			return (doallowaccess(q+1, port));
 		return (1);
 	}
 	q=buf;
@@ -832,20 +851,20 @@ int	i;
 		q += 5;
 	}
 	*q=0;
-	return (doallowaccess(buf));
+	return (doallowaccess(buf, port));
 }
 
 #else
-static int allowaccess(const RFC1035_ADDR *sin)
+static int allowaccess(const RFC1035_ADDR *sin, int port)
 {
-char	buf[RFC1035_NTOABUFSIZE];
+char	buf[RFC1035_NTOABUFSIZE+6];
 
 	rfc1035_ntoa(sin, buf);
-	return (doallowaccess(buf));
+	return (doallowaccess(buf, port));
 }
 #endif
 
-static int doallowaccess(char *buf)
+static int doallowaccess(char *buf, int port)
 {
 char	*accessptr;
 char *p, *q, *r;
@@ -854,6 +873,7 @@ int	l;
 
 	if (accessarg == 0)	return (1);
 
+	if (port) snprintf(buf+strlen(buf), 7, ".%d", ntohs(port));
 	while ((accessptr= *buf ? chkaccess(buf):0) == 0)
 	{
 		if ((accessptr=strrchr(buf, '.')) == 0
@@ -862,6 +882,12 @@ int	l;
 #endif
 			)
 		{
+			if (port)
+			{
+				snprintf(buf, 8, "*.%d", ntohs(port));
+				if ((accessptr=chkaccess(buf)) != 0)
+					break;
+			}
 			if ((accessptr=chkaccess("*")) != 0)
 				break;
 			return (1);
@@ -1146,7 +1172,24 @@ static void accepted(int n, int sockfd, RFC1035_NETADDR *sin, int sinl,
 	{
 		wait_restore(childsig);
 
-		if (allowaccess(&addr) == 0)
+		if (accesslocal) // Lookup local interface address too?
+		{
+		RFC1035_NETADDR lsin;
+		RFC1035_ADDR laddr;
+		int	lport;
+		int	i=sizeof(lsin);
+		
+			if (sox_getsockname(sockfd, (struct sockaddr *)&lsin, &i) == 0 &&
+				rfc1035_sockaddrip(&lsin, i, &laddr) == 0 &&
+				rfc1035_sockaddrport(&lsin, i, &lport) == 0 &&
+				allowaccess(&laddr,lport) == 0)
+			{
+				sox_close(sockfd);
+				_exit(0);
+			}
+		}
+
+		if (allowaccess(&addr,0) == 0)
 		{
 			if (denymsgarg) {
 				write(sockfd, denymsgarg, strlen(denymsgarg));
@@ -1385,7 +1428,7 @@ int i;
 					RFC1035_RESOLVE_RECURSIVE,
 					hostname,
 					RFC1035_TYPE_TXT,
-					RFC1035_CLASS_IN, &replyp)) >= 0)
+					RFC1035_CLASS_IN, &replyp, 0)) >= 0)
 			{
 				rfc1035_rr_gettxt(replyp->allrrs[i], 0,
 					hostname);
@@ -1405,7 +1448,7 @@ int i;
 					RFC1035_RESOLVE_RECURSIVE,
 					hostname,
 					RFC1035_TYPE_A,
-					RFC1035_CLASS_IN, &replyp)) >= 0)
+					RFC1035_CLASS_IN, &replyp, 0)) >= 0)
 			{
 				mkmymsg(varname, p->msg);
 			}
@@ -1439,7 +1482,7 @@ int i;
 			RFC1035_RESOLVE_RECURSIVE,
 			hostname,
 			hasnotxt ? RFC1035_TYPE_ANY:RFC1035_TYPE_A,
-			RFC1035_CLASS_IN, &replyp);
+			RFC1035_CLASS_IN, &replyp, 0);
 
 	if (!replyp)	return;
 
@@ -1641,6 +1684,8 @@ int rc;
 	}
 	if (accessarg && openaccess(accessarg))
 		perror(accessarg);
+	if (accesslocal && !accessarg)
+		fprintf(stderr,"-accesslocal requires -access\n");
 	rc=doit(argn, argc, argv);
 	kill( -getpid(), SIGTERM);
 	exit(rc);
